import argparse
from cryptography.fernet import Fernet
import base64
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.backends import default_backend

def create_key(password):
    bpass = password.encode('utf-8')
    kdf = PBKDF2HMAC(algorithm = hashes.SHA256(), length = 32,
                     salt = b"20200915", iterations = 100000,
                     backend=default_backend())
    key = base64.urlsafe_b64encode(kdf.derive(bpass))
    return key

def decrypt(edata, password):
    key = create_key(password)
    fn = Fernet(key)
    plain_data = fn.decrypt(edata)
    return plain_data

def encrypt(data, password):
    key = create_key(password)
    fn = Fernet(key)
    crypt_data = fn.encrypt(data)
    return crypt_data

def decrypt_string(edata, password):
    plain_data = decrypt(edata, password)
    return plain_data.decode('utf-8')

def encrypt_string(string, password):
    data = string.encode('utf-8')
    return encrypt(data, password)

def encrypt_to_file(outfile, data, password):
    encrypted = encrypt(data, password)
    with open(outfile, "wb") as encry:
        encry.write(encrypted)

def decrypt_from_file(infile, password):
    with open(infile, "rb") as encry:
        data = encry.read()
    decrypted = decrypt(data, password)
    return decrypted


# FAR rerank
# Based on W. Liu, R. Burke, Personalizing Fairness-aware Re-ranking

import argparse
import os
import re
import pandas as pd
import numpy as np
from librec_auto.core import read_config_file
from pathlib import Path


class FarHelper:
    item_feature_df = None
    protected = None
    lam = 0
    max_length = 10
    binary = True
    protected_set = {}

    def is_protected(self, itemid):
        return itemid in self.protected_set

    def num_prot(self, items):
        num_prot = [self.is_protected(itemid) for itemid in items].count(True)
        return num_prot


def entropy_(labels, base=None):
    from math import log, e
    """ Computes entropy of label distribution. """
    n_labels = len(labels)
    if n_labels <= 1:
        return 0

    value, counts = np.unique(labels, return_counts=True)
    probs = counts / n_labels
    n_classes = np.count_nonzero(probs)

    if n_classes <= 1:
        return 0
    ent = 0.
    # Compute entropy
    base = e if base is None else base
    for i in probs:
        ent -= i * log(i, base)

    return ent


def get_user_tolerance(user_profile, item_features, helper):
    # look through the items that the user has rated before
    # look through the item features and save a list of all these feature names in a file
    # call entropy_() function and return the entropy result
    user_items = user_profile['itemid'].tolist()
    if len(user_items) == 0:
        return 0
    return entropy_(item_features.loc[item_features.index.isin(user_items),
                                      'feature'].tolist())


# Caches the protected items for quicker lookup
def get_protected_set(item_features, helper):
    return set((item_features[(item_features['feature'] == helper.protected)
                              & (item_features['value'] == 1)].index).tolist())


#def is_protected(itemid):
#    item_entry = item_feature_df.loc[itemid]
#    prot_item = item_entry[(item_entry['feature']==protected) & (item_entry['value']==1)]
#    return type(prot_item) == numpy.int64

# def num_prot(items):
#     num_prot = [is_protected(itemid) for itemid in items].count(True)
#     return num_prot


def score_prot(user_profile, helper):
    user_items = user_profile['itemid'].tolist()
    if len(user_items) == 0:
        return 0
    return helper.num_prot(user_items) / len(user_items)


def rescore_binary(item, original_score, items_so_far, score_profile, user_tol,
                   helper):
    answer = original_score
    div_term = 0

    # If there are both kind of items in the list, no re-ranking happens
    count_prot = helper.num_prot(items_so_far)
    if helper.is_protected(item):
        if count_prot == 0:
            div_term = score_profile
    else:
        if count_prot == len(items_so_far):
            div_term = 1 - score_profile

    div_term *= helper.lam
    div_term *= user_tol
    answer += div_term
    return answer


# Not in the original paper, but treats the P(\\bar{s)|d) as real-valued
# See Abdollahpouri, Burke, and Mobasher. Managing popularity bias in recommender systems with personalized re-ranking. 2019
def rescore_prop(item, original_score, items_so_far, score_profile, user_tol,
                 helper):
    answer = original_score
    div_term = 0

    count_prot = helper.num_prot(items_so_far)
    count_items = len(items_so_far)
    if count_items == 0:
        div_term = score_profile
    else:
        if helper.is_protected(item):
            div_term = score_profile
            div_term *= 1 - count_prot / count_items
        else:
            div_term = (1 - score_profile)
            div_term *= count_prot / count_items

    div_term *= helper.lam
    answer += div_term
    div_term *= user_tol
    return answer


def pick_best(user_recs, user_profile, items_so_far, item_features, helper):
    best_item = None
    best_score = -1
    score_profile = score_prot(user_profile, helper)
    user_tol = get_user_tolerance(user_profile, item_features, helper)

    for _, _, item, score in user_recs.itertuples():
        if helper.binary:
            new_score = rescore_binary(item, score, items_so_far,
                                       score_profile, user_tol, helper)
        else:
            new_score = rescore_prop(item, score, items_so_far, score_profile,
                                     user_tol, helper)
        if new_score > best_score:
            best_item = item
            best_score = new_score

    return (best_item, best_score)


def rerank(userid, user_recs_df, user_profile, item_features, helper):
    output_data = []
    items_so_far = []

    for i in range(0, helper.max_length):

        item, score = pick_best(user_recs_df, user_profile, items_so_far,
                                item_features, helper)

        items_so_far.append(item)
        output_data.append((userid, item, score))
        new_user_recs = user_recs_df[user_recs_df['itemid'] != item]
        user_recs_df = new_user_recs

    return pd.DataFrame(output_data, columns=['userid', 'itemid', 'score'])


def execute(recoms_df, train_df, item_features, helper):
    result = []

    for userid in list(set(recoms_df['userid'])):
        #        print('list reranked for user #',userid)
        result.append(
            rerank(userid, recoms_df[recoms_df['userid'] == userid].copy(),
                   train_df[train_df['userid'] == userid], item_features,
                   helper))

    rr_df = pd.concat(result)
    return rr_df


def read_args():
    """
    Parse command line arguments.
    :return:
    """
    parser = argparse.ArgumentParser(description='Generic re-ranking script')
    parser.add_argument('conf', help='Name of configuration file')
    parser.add_argument('target', help='Experiment target')
    parser.add_argument('original', help='Path to original results directory')
    parser.add_argument('result', help='Path to destination results directory')
    parser.add_argument(
        '--max_len',
        help='The maximum number of items to return in each list',
        default=10)
    parser.add_argument('--lambda', help='The weight for re-ranking.')
    parser.add_argument('--binary',
                        help='Whether P(\\bar{s)|d) is binary or real-valued',
                        default=True)

    input_args = parser.parse_args()
    return vars(input_args)


RESULT_FILE_PATTERN = 'out-\d+.txt'
INPUT_FILE_PATTERN = 'cv_\d+'


def enumerate_results(result_path):
    files = os.listdir(result_path)
    pat = re.compile(RESULT_FILE_PATTERN)
    return [file for file in files if pat.match(file)]


if __name__ == '__main__':
    args = read_args()
    #print(args)
    config = read_config_file(args['conf'], args['target'])
    result_files = enumerate_results(args['original'])

    split_path = config.get_files().get_split_path()
    # split_names = os.listdir(split_path)

    data_dir = config.get_prop_dict()['dfs.data.dir']
    item_feature_file = config.get_prop_dict()['data.itemfeature.path']
    protected = config.get_prop_dict()['data.protected.feature']

    item_feature_path = Path(data_dir) / item_feature_file

    item_feature_df = None

    if not item_feature_path.exists():
        print("Cannot locate item features. Path: " + item_feature_path)
        exit(-1)
    else:
        item_feature_df = pd.read_csv(item_feature_path,
                                      names=['itemid', 'feature', 'value'])
        item_feature_df.set_index('itemid', inplace=True)

    helper = FarHelper()
    helper.protected_set = get_protected_set(item_feature_df, helper)
    helper.lam = float(args['lambda'])
    helper.max_length = int(args['max_len'])
    helper.binary = args['binary'] == 'True'
    helper.protected = protected

    for file_name in result_files:

        # reading the result
        input_file_path = Path(args['original'] + '/' + file_name)

        # reading the training set
        cv_path = str(split_path) + '/cv_' + re.findall(
            '\d+', file_name)[0] + '/train.txt'
        tr_file_path = Path(cv_path)

        tr_df = None
        if tr_file_path.exists():
            tr_df = pd.read_csv(tr_file_path,
                                names=['userid', 'itemid', 'score'],
                                sep='\t')
        else:
            print('Cannot locate training data: ' + tr_file_path)
            exit(-1)

        if input_file_path.exists():
            recoms_df = pd.read_csv(input_file_path,
                                    names=['userid', 'itemid', 'score'])

            reranked_df = execute(recoms_df, tr_df, item_feature_df, helper)

            output_file_path = Path(args['result'] + '/' + file_name)
            print('Reranking for ', output_file_path)
            reranked_df.to_csv(output_file_path, header=None, index=False)



'''
Author: Zijun Liu
Description:
       This script is used to extract the experimental parameters and store it into csv
'''
import argparse
from librec_auto.core import read_config_file
from librec_auto.core.util import Status
from collections import defaultdict

import pandas as pd
import numpy as np

extract_params_format = "extract_{}_params"

def get_metric_info(files):
    metric_info = {}
    for sub_paths in files.get_sub_paths_iterator():
        status = Status(sub_paths)
        if status.is_completed():
            params = status._params
            vals = status._vals
            log = status._log
            metric_info[status._name] = (params, vals, log)
    return metric_info

def extract_full_info(config):
    metric_info = get_metric_info(config.get_files())
    exp_frames = []
    table_values = defaultdict(list)
    time_stamp = None

    for exp in metric_info.keys():
        params, vals, log = metric_info[exp]
        time_stamp = log.get_time_stamp()
        entry_count = log.get_kcv_count()
        table_values['Experiment'] = np.repeat(exp, entry_count)
        table_values['Split'] = range(0, entry_count)

        for (param, val) in zip(params, vals):
            table_values[param] = np.repeat(val, entry_count)

        for metric in log.get_metrics():
            table_values[metric] = log.get_metric_values(metric)[0:entry_count]

        exp_df = pd.DataFrame(table_values)
        exp_frames.append(exp_df)

    exp_results = pd.concat(exp_frames, axis=0, ignore_index=True)
    return (exp_results, time_stamp)


def extract_summary_info(config):
    metric_info = get_metric_info(config.get_files())
    table_values = defaultdict(list)
    time_stamp = None

    for exp in metric_info.keys():
        params, vals, log = metric_info[exp]
        time_stamp = log.get_time_stamp()
        table_values['Experiment'].append(exp)

        for (param, val) in zip(params, vals):
            table_values[param].append(val)

        for metric in log.get_metrics():
            table_values[metric].append(log.get_metric_values(metric)[-1])

    exp_results = pd.DataFrame(table_values)
    return (exp_results, time_stamp)


def save_data(df, choice, time_stamp):
    save_path = config.get_files().get_post_path() / f'study-results-{choice}_{time_stamp}.csv'
    df.to_csv(save_path, index=False)

def read_args():
    """
    Parse command line arguments.
    :return:
    """
    parser = argparse.ArgumentParser(description="Extract the experimental parameters")
    parser.add_argument('conf', help='Path to configuration file')
    parser.add_argument('target', help='Experiment target')
    parser.add_argument('--option', help='Type of csv', choices=['summary', 'full', 'all'])
    input_args = parser.parse_args()
    return vars(input_args)

if __name__ == '__main__':
    args = read_args()
    config = read_config_file(args['conf'], args['target'])

    choice = args['option']

    if choice == "summary":
        df, time_stamp = extract_summary_info(config)
        save_data(df, choice, time_stamp)
    elif choice == "full":
        df, time_stamp = extract_full_info(config)
        save_data(df, choice, time_stamp)
    elif choice == "all":
        df, time_stamp = extract_summary_info(config)
        save_data(df, "summary", time_stamp)
        df, time_stamp = extract_full_info(config)
        save_data(df, "full", time_stamp)

    else:
        print ('Unrecognized option for results_to_csv')



import dropbox as db
import argparse
from librec_auto.core.util.encrypt import decrypt_from_file
import getpass
from pathlib import Path

def dropbox_send_file(filepath, destpath, api_key):
    dropbox_api = db.Dropbox(api_key)
    filename = filepath.name
    destname = destpath / filename
    with open(filepath, "rb") as f:
        dropbox_api.files_upload(f.read(), str(destname),
                                 db.files.WriteMode.overwrite, mute=False)

def dropbox_send_folder(filepath, destpath, api_key):
    dropbox_api = db.Dropbox(api_key)
    for file in filepath.iterdir():
        destname = destpath / file.name
        with open(file) as f:
            dropbox_api.files_upload(f.read(), str(destname),
                                     db.files.WriteMode.overwrite, mute=True)

'''
Zijun:
    About how to run this file. 
    There are Three options for the Dropbox to recognize. 
    file:
        Description: Post single file to dropbox
        give repo to single file
    folder:
        Description: Post entire folder to dropbox
        give repo to folder to Dropbox
'''


def read_args():
    parser = argparse.ArgumentParser(description="Dropbox Post-Processing script")
    parser.add_argument('conf', help='Path to configuration file')
    parser.add_argument('target', help='Experiment target')
    parser.add_argument("--option", help='Which actions you want to do', choices=["file", "folder", "No"])
    parser.add_argument('--encrypted_key', help="Encrpyted slack api key file")
    parser.add_argument('--path', help="File/folder to post")
    parser.add_argument('--dest', help="Destination file/folder")
    parser.add_argument('--password', help="Password to encrypted API key")

    input_args = parser.parse_args()
    return vars(input_args)


if __name__ == '__main__':
    args = read_args()
    action = args["option"]

    encrypted_file = args['encrypted_key']
    filename = args['path']
    destname = args['dest']

    pw = args['password']
    if pw == None:
        pw = getpass.getpass(f'Password for {encrypted_file}:')

    api_key_bytes = decrypt_from_file(encrypted_file, pw)
    api_key = api_key_bytes.decode('utf-8')

    filepath = Path(filename)
    destpath = Path(destname)

    if action == "file":
        dropbox_send_file(filepath, destpath, api_key)
    elif action == "folder":
        dropbox_send_folder(filepath, destpath, api_key)
    elif action == "No":
        exit()



import argparse
from librec_auto.core import read_config_file
from librec_auto.core.util import Status
import webbrowser

import matplotlib
import matplotlib.pyplot as plt

matplotlib.use('Agg')  # For non-windowed plotting

viz_file_pattern_bar = "viz-bar-{}.jpg"
viz_file_pattern_box = "viz-box-{}.jpg"
viz_html_filename = "viz.html"
exp_dir_pattern = "exp[0-9][0-9][0-9]"

# Things we need from each experiment run
# name (dir), params that changed, values of changing parameters, metrics measured,


def get_metric_info(files):
    metric_info = {}

    for sub_paths in files.get_sub_paths_iterator():
        status = Status(sub_paths)

        if status.is_completed():
            params = status._params
            vals = status._vals
            log = status._log

            metric_info[status._name] = (params, vals, log)

    return metric_info


def create_bar(path, metric_name, params, settings, metric_values):
    x_range = range(0, len(settings))

    fig, ax = plt.subplots()
    ax.bar(x_range, metric_values, width=0.3)
    ax.set_ylabel(metric_name)
    ax.set_title('{} by\n{}'.format(metric_name, params))
    ax.set_xticks(x_range)
    ax.set_xticklabels(settings)

    filename = path / viz_file_pattern_bar.format(metric_name)
    fig.savefig(str(filename))
    plt.close()

    return filename


def create_bars(path, metric_info):
    metric_names = list(metric_info.values())[0][2].get_metrics()
    # Nasim: add list to it because in python 3 it returns a view, so it doesn't have indexing, you can't access it.

    bar_paths = []

    for metric in metric_names:

        param_string = ""
        settings = []
        metric_vals = []

        for params, vals, log in metric_info.values():
            param_string = ', '.join(params)
            settings.append('\n'.join(vals))
            metric_vals.append(float(log.get_metric_values(metric)[-1]))

        bar_paths.append(
            create_bar(path, metric, param_string, settings, metric_vals))

    return bar_paths


def create_box(path, metric, params, settings, fold_values):
    fig, ax = plt.subplots()
    ax.boxplot(fold_values)
    ax.set_ylabel(metric)
    ax.set_xticklabels(settings)
    ax.set_title('{} distribution by\n{}'.format(metric, params))

    filename = path / viz_file_pattern_box.format(metric)

    fig.savefig(str(filename))
    plt.close()
    return filename


def create_boxes(path, metric_info):
    metric_names = list(metric_info.values())[0][2].get_metrics()

    box_paths = []
    for metric in metric_names:

        param_string = ""
        settings = []
        fold_vals = []

        for params, vals, log in metric_info.values():
            param_string = ', '.join(params)
            settings.append('\n'.join(vals))
            metric_vals = log.get_metric_values(metric)[:-1]
            fold_vals.append([float(val) for val in metric_vals])

        box_paths.append(
            create_box(path, metric, param_string, settings, fold_vals))

    return box_paths


PAGE_TEMPLATE = '<html><h1>Study results</h1>{}</html>'
METRIC_TEMPLATE = '<h2>Metric: {}</h2>{}'
IMAGE_TEMPLATE = '<img src="{}" />'


def create_html(path, metric_info, bars, boxes):
    html = PAGE_TEMPLATE
    metric_chunks = []
    metric_names = list(metric_info.values())[0][2].get_metrics()

    if boxes is None:
        for name, bar in zip(metric_names, bars):
            images = IMAGE_TEMPLATE.format(bar.name)
            metric_chunks.append(METRIC_TEMPLATE.format(name, images))
    else:
        for name, bar, box in zip(metric_names, bars, boxes):
            images = IMAGE_TEMPLATE.format(bar.name) + IMAGE_TEMPLATE.format(
                box.name)
            metric_chunks.append(METRIC_TEMPLATE.format(name, images))

    output = html.format('\n'.join(metric_chunks))

    filename = path / viz_html_filename

    with open(filename, 'w') as out_file:
        out_file.write(output)

    return filename


def create_graphics(config, display):
    files = config.get_files()
    metric_info = get_metric_info(files)

    print("Post path")
    print(files.get_post_path())

    bars = create_bars(files.get_post_path(), metric_info)

    if config.cross_validation() > 1:       # Box plot only makes sense for cross-validation
        boxes = create_boxes(files.get_post_path(), metric_info)
    else:
        boxes = None

    if display:
        html_file = create_html(files.get_post_path(), metric_info, bars,
                                boxes)
        webbrowser.open('file://' + str(html_file.absolute()),
                        new=1,
                        autoraise=True)


def read_args():
    """
    Parse command line arguments.
    :return:
    """
    parser = argparse.ArgumentParser(
        description='Generic post-processing script')
    parser.add_argument('conf', help='Path to configuration file')
    parser.add_argument('target', help='Experiment target')
    parser.add_argument('--browser',
                        help='Show graphics in browser',
                        choices=['true', 'false'])

    input_args = parser.parse_args()
    return vars(input_args)


if __name__ == '__main__':
    args = read_args()
    config = read_config_file(args['conf'], args['target'])

    print(f"librec-auto: Creating summary visualizations for {args['target']}")

    display = args['browser'] == 'true'

    create_graphics(config, display)



import slack as sl
import argparse
from librec_auto.core.util.encrypt import decrypt_from_file
import getpass


def slack_send_message(message, channel, slack_api):
    client = sl.WebClient(token=slack_api)
    client.chat_postMessage(channel=channel, text=message)

def slack_send_file(filename, message, channel, slack_api):
    client = sl.WebClient(token=slack_api)
    with open(filename, 'rb') as att:
        r = client.api_call("files.upload", files={'file': att, },
                            data={'channels': channel, 'filename': filename, 'title': f'File: {filename}',
                                  'initial_comment': message}
                            )
    assert r.status_code == 200

'''
Zijun:
    About how to run this file. 
    There are two action for the slack to recognize. The first one is the message. The second one is the file

    message:
        Description: If you want to direct post a sentence in to the slack channel. 
        The command should like this:
        python slack_post.py [action] [API-Token] ["Expression"]
        eg: python slack_post.py message xoxb-625375728051-787189332448-Ufqx4usZgQUW9c3q3J0JsH7o "Hello World"

    file: 
        Description: If you want to post file into slack.
        The command should like this:
        python slack_post.py [action] [API-Token] /directory
        eg: python slack_post.py file xoxb-625375728051-787189332448-Ufqx4usZgQUW9c3q3J0JsH7o /directory/---.py

'''


def read_args():
    parser = argparse.ArgumentParser(description="Slack Post-Processing script")
    parser.add_argument('conf', help='Path to configuration file')
    parser.add_argument('target', help='Experiment target')
    parser.add_argument("--option", help='Which actions you want to do', choices=["message", "file", "No"])
    parser.add_argument('--channel', help='Which channel you want to post')
    parser.add_argument('--encrypted_key', help="Encrpyted slack api key file")
    parser.add_argument('--file', help="File to post")
    parser.add_argument('--password', help="Password to encrypted API key")
    parser.add_argument('--message', help="Message to post")

    input_args = parser.parse_args()
    return vars(input_args)


if __name__ == '__main__':
    args = read_args()
    action = args["option"]

    channel = args['channel']
    encrypted_file = args['encrypted_key']
    filename = args['file']
    message = args['message']

    pw = args['password']
    if pw == None:
        pw = getpass.getpass(f'Password for {encrypted_file}:')

    api_key_bytes = decrypt_from_file(encrypted_file, pw)
    api_key = api_key_bytes.decode('utf-8')

    if action == "message":
        slack_send_message(message, channel, api_key)
    elif action == "file":
        slack_send_file(filename, message, channel, api_key)
    elif action == "No":
        exit()





import datetime
from pathlib import Path
from librec_auto.core.util import xml_load_from_path, force_list, SubPaths, LogFile
from librec_auto.core.util.xml_utils import single_xpath
from lxml import etree

# A .status file looks like this
# <librec-auto-status>
#    <message>Completed</message>
#    <exp-no>1</exp-no>\
#    <param><name>rec.neighbors.knn.number</name><value>30</value></param>
#    <date>June 28, 11:00 PM</date>
# </librec-auto-status>
# TODO: Rewrite with lxml. This is kind of embarrassing.


class Status():

    def __init__(self, sub_paths):
        self._subpaths = sub_paths
        status_path = self._subpaths.get_path('status')

        if status_path.exists():
            self._name = sub_paths.subexp_name
            self._status_xml = xml_load_from_path(status_path)
            self._message = single_xpath(self._status_xml, '/librec-auto-status/message').text

            if self._subpaths.get_path('log').exists():
                self._log = LogFile(self._subpaths)
            else:
                self._log = None

            params = self._status_xml.xpath('//param')
            if params != None:
                self.process_params(params)
            else:
                self._params = []
                self.m_vals = []

    def __str__(self):
        params_string = self.get_params_string()
        if self._log:
            results_string = self.get_log_info()
        else:
            results_string = "No LibRec results"
        return f'Status({self._name}:{self._message}{params_string} Overall{results_string})'

    def is_completed(self):
        return self._message == 'Completed'

    def process_params(self, param_elements):
        param_list = []
        val_list = []

        # Maybe could be done with an ugly list comprehension
        for param_xml in param_elements:
            name_elem = single_xpath(param_xml, 'name')
            param_list.append(name_elem.text)
            val_elem = single_xpath(param_xml, 'value')
            val_list.append(val_elem.text)

        self._params = param_list
        self._vals = val_list

    def get_params_string(self):
        params_string = ''
        if self._params == []:
            return " No parameters"
        for param, val in zip(self._params, self._vals):
            params_string += f' {param}: {val}'
        return params_string

    def get_log_info(self):
        kcv_count = self._log.get_kcv_count()
        if kcv_count is None:
            return self.get_metric_info(self._log, 0)
        else:
            return self.get_metric_info(self._log, -1)

    def get_metric_info(self, log, idx):
        metric_info = ''
        for metric_name in log.get_metrics():
            metric_value = log.get_metric_values(metric_name)[idx]
            metric_info = metric_info + f' {metric_name}: {float(metric_value):.3f}'
        return metric_info

    @staticmethod
    def save_status(msg, exp_count, config, paths):
        status_xml = etree.Element("librec-auto-status")
        status_xml.append(etree.Comment('DO NOT EDIT. File automatically generated by librec-auto'))
        msg_elem = etree.SubElement(status_xml, "message")
        msg_elem.text = msg
        expno_elem = etree.SubElement(status_xml, "exp_no")
        expno_elem.text = str(exp_count)
        date_elem = etree.SubElement(status_xml, "date")
        date_elem.text = str(datetime.datetime.now())

        conf_xml = config.get_files().get_sub_paths(exp_count).get_exp_conf()
        var_elems = conf_xml.xpath("//*[@var='true']")
        for var_elem in var_elems:
            if var_elem.tag == 'param':
                var_name = var_elem.get('name')
            else:
                var_name = var_elem.tag
            var_value = var_elem.text

            param_elem = etree.SubElement(status_xml, "param")

            name_elem = etree.SubElement(param_elem, "name")
            name_elem.text = var_name
            value_elem = etree.SubElement(param_elem, "value")
            value_elem.text = var_value

        status_file = paths.get_path('status')

        status_xml.getroottree().write(status_file.absolute().as_posix(), pretty_print=True)

    # # Accept list of vars and tuples
    # @staticmethod
    # def save_status(msg, exp_count, config, paths):
    #     status_file = paths.get_path('status')
    #     status_front = Status._TEMPLATE_FRONT.format(msg, exp_count, datetime.datetime.now())
    #
    #     status_params = ''
    #     conf_xml = config.get_files().get_sub_paths(exp_count).get_exp_conf()
    #     var_elems = conf_xml.xpath("//*[@var='true']")
    #     for var_elem in var_elems:
    #         if var_elem.tag == 'param':
    #             var_name = var_elem.get('name')
    #         else:
    #             var_name = var_elem.tag
    #         var_value = var_elem.text
    #
    #         status_params = status_params + Status._TEMPLATE_LINE.format(var_name, var_value)
    #
    #     status_info = Status._HEADER + status_front + status_params + Status._TEMPLATE_END
    #
    #     with status_file.open(mode='w') as fh:
    #         fh.write(str(status_info))
